#include <LiquidCrystal.h>

#include <SD.h>

#include <SPI.h>

#include <Wire.h>
#include "RTClib.h"

// folgende Definitionen werden im Programm verwendet.
#define UP A0                                 // eine Stelle nach oben im StringArray
#define DOWN A1                               // eine Stelle nach unten im StringArray
#define SET A2                                // Ausgewählte Stelle im StringArray übernehmen in eine Variable
#define ENTER A3                              // Taster zum Speichern des Datensatzes auf SD-Karte
#define CHIP_SELECT 8                         // definiert den Pin-A4 am Arduino als CHIP_SELECTPin fuer das Schreiben auf die SD-Karte
#define DCF_PIN 2                             // festlegen des Interrupt Pins für das DCF77 Modul 
#define DCF_INTERRUPT 0                       // wird für die DCF Uhr benötigt. 
//
LiquidCrystal lcd(0, 1, 3, 4, 5, 6, 7);       // LCD Display Pinbelegung Steuerung-Pin: 0 1 3  Datenbits D1-4 Pin: 4 5 6 7
//
File myLog;                                   // erstellen des Objektes myLog zum Zugriff auf die Logbuchdatei auf der SD Karte
//
RTC_DS1307 rtc;
String strStunde, strMinute, strTag, strMonat, strJahr; // Globale Variablen für die Zeit zum Schreiben des ADIF - Datensatz und Zeitanzeige im LCD
boolean boSpeichern = true;                   // Speichervariable, wird vom Button Enter genutzt um die Eingabe zu beenden und zu speichern
//
void setup() {
  // put your setup code here, to run once:
  Wire.begin();
  rtc.begin();
  if (! rtc.isrunning()) {
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
  //
  // die Taster UP, DOWN, SET, ENTER als INPUT mit PULLUP auf +5V gegen Masse dann LOW
  pinMode(UP, INPUT_PULLUP);          // eine Stelle hoch im jeweiligen StringArray
  pinMode(DOWN, INPUT_PULLUP);        // eine Stelle nach unten im jeweiligen StringArray
  pinMode(SET, INPUT_PULLUP);         // Übernahme des Strings aus dem StringArray und wechseln zur nächseten Stelle im Rufzeichen
  pinMode(ENTER, INPUT_PULLUP);       // Speichern des Rufzeichens als ADIF String
  pinMode(CHIP_SELECT, OUTPUT);       // setzt den Pin fuer CS der SD-Karte als Ausgang
  lcd.begin(8, 2);                    // Initialisieren des LCD 16 Zeichen, eine Zeile, Typ-A hier 2x8 Zeichen, es gibt auch Typ-B = 1x16
  lcd.print("SyncTime");              // Ausgabe Sync Time blinkend mit delay und clear lcd, zur Anzeige das die Zeit noch synchronisiert wird.
  delay(500);
  lcd.clear();
  delay(500);
  Serial.begin(9600);
}
int getFreeRAM() 
{
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}


void loop() {
 // Hauptprogramm
  //
  Serial.println(getFreeRAM());
  //if (now() != prevDisplay) { // schreiben des LCD wenn sich die Zeit um eine Sekunde geändert hat,
    //                        // das hat den Vorteil das am Ende vom Hauptprogramm ein kleineres Delay gewählt werden kann zur Tasterabfrage
    //prevDisplay = now();      // setzen der Variable zur Prüfung der Zeitänderung
    //
    // Schreiben der Zeit im LCD Aktualisierung jede Sekunde (siehe dazu Delay am Ende des Hauptprogramms.)
    DateTime now = rtc.now();
    strStunde = String(now.hour());
    strMinute = String(now.minute());
    strTag = String(now.day());
    strMonat = String(now.month());
    lcd.setCursor(0, 0);      // Cursor nach ganz links setzen
    lcd.print(strStunde);     // Inhalt der Variablen Stunde schreiben.
    lcd.print(":");           //
    lcd.print(strMinute);     // Inhalt der Variablen Minute schreiben.
    lcd.print(":");           // Schreiben der Sekunden nur auf das LCD, wenn die Sekunde kleiner als 10 dann mit führender Null
    if (now.second() < 10) {
      lcd.print('0');
      lcd.print(now.second());
    } else {
      lcd.print(now.second());   // Inhalt der Variablen Sekunde schreiben.
    }
    lcd.setCursor(2, 1);     // Setzen des Cursor in die zweiten 8 Zeichen des LCD siehe oben in Setup Typ A und Typ B
    lcd.print(strTag);       // Inhalt der Variablen Tag schreiben
    lcd.print(".");          //
    lcd.print(strMonat);     // Inhalt der Variablen Monat schreiben
    
  //}

  //
  // Beginn der Prüfung ob ein Taster UP oder Down gedrückt wurde,
  // dient hier auch zur Ermittlung in welchem UKW Band das QSO geführt wurde 2m oder 70cm
  // alle Taster liegen auf HIGH, weil an Pullup gelegt, deshalb hier Prüfung auf LOW Signal da die Taster Schließer sind
  if (digitalRead(UP) == LOW) {
    //
    lcd.clear();                    // löschen vom LCD -> Bereitschaft für Rufzeicheneingabe
    schreibeDaten("<band:2>2M");    // Aufruf des Unterprogramms schreibe Daten mit Übergabe des ADIF formatierten Strings mit dem Afu Band
  }
  if (digitalRead(DOWN) == LOW) {
    //;
    lcd.clear();                    // löschen vom LCD -> Bereitschaft für Rufzeicheneingabe
    schreibeDaten("<band:4>70cm");  // Aufruf des Unterprogramms schreibe Daten mit Übergabe des ADIF formatierten Strings mit dem Afu Band
  }
  //
  delay(180);                       // alle 180ms werden die Taster abgefragt

}
